@@ -4,19 +4,20 @@ import os
 import functools
 import distutils.core
 import distutils.filelist
-from distutils.core import Command as _Command
 from distutils.util import convert_path
 from fnmatch import fnmatchcase
 
+from setuptools.extern.six.moves import filterfalse, map
+
 import setuptools.version
 from setuptools.extension import Extension
-from setuptools.dist import Distribution, Feature, _get_unpatched
+from setuptools.dist import Distribution, Feature
 from setuptools.depends import Require
-from setuptools.compat import filterfalse
+from . import monkey
 
 __all__ = [
     'setup', 'Distribution', 'Feature', 'Command', 'Extension', 'Require',
-    'find_packages'
+    'find_packages',
 ]
 
 __version__ = setuptools.version.__version__
@@ -31,6 +32,7 @@ lib2to3_fixer_packages = ['lib2to3.fixes']
 
 
 class PackageFinder(object):
+
     @classmethod
     def find(cls, where='.', exclude=(), include=('*',)):
         """Return a list all Python packages found within directory 'where'
@@ -107,16 +109,20 @@ class PackageFinder(object):
         """
         return lambda name: any(fnmatchcase(name, pat=pat) for pat in patterns)
 
+
 class PEP420PackageFinder(PackageFinder):
+
     @staticmethod
     def _looks_like_package(path):
         return True
 
+
 find_packages = PackageFinder.find
 
 setup = distutils.core.setup
 
-_Command = _get_unpatched(_Command)
+_Command = monkey.get_unpatched(distutils.core.Command)
+
 
 class Command(_Command):
     __doc__ = _Command.__doc__
@@ -136,9 +142,6 @@ class Command(_Command):
         vars(cmd).update(kw)
         return cmd
 
-# we can't patch distutils.cmd, alas
-distutils.core.Command = Command
-
 
 def _find_all_simple(path):
     """
@@ -164,5 +167,4 @@ def findall(dir=os.curdir):
     return list(files)
 
 
-# fix findall bug in distutils (http://bugs.python.org/issue12885)
-distutils.filelist.findall = findall
+monkey.patch_all()
